from ipkiss3 import all as i3
from ring import RingResonator


# PCell containing two rings.
class TwoRings(i3.PCell):
    """Parametric cell with two rings defined by the user, which are stacked
    vertically, with the bottom one flipped.
    """

    _name_prefix = "TWORING"

    # 1.
    # The rings are defined as ChildCellProperties. This is a special property
    # that for hierarchical PCells. In each view of the PCell of a certain type (e.g. Layout)
    # the corresponding viewtype (e.g. Layout) of the ChildCell will automatically available.

    ring_1 = i3.ChildCellProperty()
    ring_2 = i3.ChildCellProperty()

    # Layout view
    class Layout(i3.LayoutView):
        def _generate_instances(self, insts):
            # 1. calculate the transformations of the rings based on their properties
            t1 = i3.Translation((0.0, self.ring_1.ring_radius + 2.0))
            t2 = i3.VMirror() + i3.Translation((0.0, -self.ring_2.ring_radius - 5.0))

            # 2. Generating the instances

            # self.ring1 refers to the layout view of ring1. Here is instantiated in execute.py
            insts += i3.SRef(name="ring_1", reference=self.ring_1, transformation=t1)
            # self.ring1 refers to the layout view of ring1. Here is instantiated in execute.py
            insts += i3.SRef(name="ring_2", reference=self.ring_2, transformation=t2)

            return insts


# PCell containing two rings.
class TwoRingsWithDefaults(i3.PCell):
    """
    Parametric cell with two rings. If the user does not provide the rings, they are created automatically
    using default methods.
    """

    _name_prefix = "TWORING"

    # 1.
    # The rings are defined as ChildCellProperties. This is a special property
    # that for hierarchical PCells. In each view of the PCell of a certain type (e.g. Layout)
    # the corresponding viewtype (e.g. Layout) of the ChildCell will automatically available.

    ring_1 = i3.ChildCellProperty()
    ring_2 = i3.ChildCellProperty()

    # Here, we provide a default value for ring_1
    def _default_ring_1(self):
        return RingResonator(name=f"{self.name}_ring1")

    # Here, we provide a default value for ring_2
    def _default_ring_2(self):
        return RingResonator(name=f"{self.name}_ring2")

    # Layout view
    class Layout(i3.LayoutView):
        # Here, we provide a default value for ring_1
        def _default_ring_1(self):
            ring_1 = self.cell.ring_1.get_default_view(
                i3.LayoutView
            )  # Retrieve the layout view of ring_1 of same type (i3.LayoutView)
            ring_1.ring_radius = 5.0  # Set the ring radius to 5.0
            return ring_1

        # Here, we provide a default value for ring_2
        def _default_ring_2(self):
            ring_2 = self.cell.ring_2.get_default_view(
                i3.LayoutView
            )  # Retrieve the layout view of ring_2 of same type (i3.LayoutView)
            ring_2.ring_radius = 6.0  # Set the ring radius to 6.0
            return ring_2

        def _generate_instances(self, insts):
            # 1. calculate the transformations of the rings based on their properties
            t1 = i3.Translation((0.0, self.ring_1.ring_radius + 2.0))
            t2 = i3.VMirror() + i3.Translation((0.0, -self.ring_2.ring_radius - 5.0))

            # 2. Generating the instances
            # self.ring1 refers to the layout view of ring1. Here is instantiated in execute.py
            insts += i3.SRef(name="ring_1", reference=self.ring_1, transformation=t1)
            # self.ring1 refers to the layout view of ring1. Here is instantiated in execute.py
            insts += i3.SRef(name="ring_2", reference=self.ring_2, transformation=t2)

            return insts
